@@ -28,12 +28,11 @@ from typing import (
 )
 
 
-from wechaty_puppet import FileBox, ImageType   # type: ignore
+from wechaty_puppet import (    # type: ignore
+    FileBox, ImageType, get_logger
+)
 
 from ..accessory import Accessory
-from ..config import (
-    get_logger,
-)
 
 log = get_logger('Image')
 
@@ -54,7 +53,7 @@ class Image(Accessory):
         :param image_id:
         """
         super().__init__()
-        log.info('__init__(%d)', image_id)
+        log.info('init the message Image object <%s>', image_id)
 
         self.image_id = image_id
         if self.puppet is None:
